PRO HWIND
compile_opt hidden

;HWINDZ.M
;...generate complex array of mesoscale wind components after processing krdfile
KRD2ZUVZ,xxobs,yxobs,uxobs,vxobs,noenh,xwst,ysth,dxyk,elo,gglo,ofname,zuv,zo,xmkm,ynkm,levelzo,n_big
n_big=n_big/2

;...determine location and value of enhanced wind maximum:
zxobs=complex(xxobs,yxobs)-zo & rxobs=float(sqrt(zxobs*conj(zxobs)))
wxobs=sqrt(uxobs^2+vxobs^2)

;...generate mesoscale coordinate grid
ZGRIDMESO,zuv,xmkm,ynkm,rxmeso,wxmeso,zgrid
ijc=where(abs(xmkm) eq min(abs(xmkm)))
zuv(ijc,ijc)=complex(0,0) ;mesoscale wind center has zero wind

;...determine if enhancement will take place for this analysis
if rxobs gt rxmeso then rxobs=rxmeso
;Here I am commenting the next line, since the wxobs should be the highest
;if wxmeso gt wxobs then wxobs=wxmeso
if wxobs lt 15 or rxmeso gt 5*rxobs then noenh=1
if noenh eq 1 then begin rxobs=rxmeso & wxobs=wxmeso &endif
xl=abs(min(xmkm)) & xr=max(xmkm) & yb=abs(min(ynkm)) & yt=max(ynkm)
radx=min([xl,xr,yb,yt]) & if 2*rxmeso ge radx then begin $
rxobs=rxmeso & wxobs=wxmeso & endif

;...compute enhanced (boosted and contracted) winds
zuvbc=zuv & if fix(noenh) ne 1 then $
ENHANCEX,wxobs,wxmeso,rxobs,rxmeso,zuv,zuvbc,zgrid,dxyk

;...reconcile wind center and observed center coordinates and compute latitude and longitude
XYTOLL,1000*xmkm,1000*ynkm,elon,nlat,0.,0.
elon=0.5*(elon+nlat) & nlat=elon+gglo & elon=elon+elo

;...generate landmask for open terrain winds
LANDMASK,elon,nlat,lmsk

;...generate open terrain winds from zuvbc and merge with the enhanced marine winds
ZUV2ZOT,zuvbc,zotbc & ZUVMRG,zuvbc,zotbc,lmsk,zmrg
if levelzo ne 19 then zmrg=zuvbc

;...generate the gridded field of enhanced and merged wind components
GRIDGEN,ofname,1000*dxyk,xmkm,ynkm,elo,gglo,elon,nlat,zmrg

;...calculate position and values of maximum wind components
wmeso=float(sqrt(zuv*conj(zuv))) & wmare=float(sqrt(zuvbc*conj(zuvbc))) & wmrg=float(sqrt(zmrg*conj(zmrg)))
imeso=where(wmeso eq max(wmeso)) & imare=where(wmare eq max(wmare)) & imrg=where(wmrg eq max(wmrg))
zmeso=zgrid(imeso) & zmare=zgrid(imare) & zmge=zgrid(imrg)
zuvmeso=zuv(imeso) & zuvmare=zuvbc(imare) & zuvmrg=zmrg(imrg)

;...calculate enhanced marine wind amplitudes
ZUV2ZA,zuvbc,zamare & zaname='enhanced_amp'

;...generate wind radii table
QRADGY,1.94*wmrg,zgrid,qrad

;...write file for entry into database
AMPGENX,zaname,dxyk,xmkm,ynkm,elo,gglo,elon,nlat,zamare,qrad,zmeso,zuvmeso,zmare,zuvmare,zmge,zuvmrg

;...generate the background field for later analyses (lf format)
date=123456. & time=long(123456) & WRBACK,zgrid,zuvbc,date,time,0.,0.
;END HWINDZ.M

;STATFILE.M
;... acquire actual observed wind components and thier storm-relative locations
RDOBS,'anal',ntp,xkm,ykm,rkm,azi,ums,vms,wms,elo,gglo
ine50=where(ntp ne 50)
azio=azi(ine50) & ntpo=ntp(ine50) & rkmo=rkm(ine50)
umso=ums(ine50) & vmso=vms(ine50) & wmso=wms(ine50) & xkmo=xkm(ine50) & ykmo=ykm(ine50)
wxobs=max(wmso)
iwxobs=where(wmso eq wxobs)
xxobs=xkmo(iwxobs) & yxobs=ykmo(iwxobs)
uxobs=umso(iwxobs) & vxobs=vmso(iwxobs)
sobs=rkmo/rxobs(0)

inrge=where(xkmo ge min(xmkm) and xkmo le max(xmkm) and ykmo ge min(ynkm) and ykmo le max(ynkm)) 
ntp=ntpo(inrge) & xkm=xkmo(inrge) & ykm=ykmo(inrge)
ums=umso(inrge) & vms=vmso(inrge) & wms=wmso(inrge)
ixkm=(xkm-min(xmkm))/dxyk & jykm=(ykm-min(ynkm))/dxyk

;... evaluate the analyzed windfield at the locations of the observations
umare=float(zuvbc) & vmare=imaginary(zuvbc)
umsa=interpolate(umare,ixkm,jykm,cubic=-.05)
vmsa=interpolate(vmare,ixkm,jykm,cubic=-.05)
wmsa=interpolate(wmare,ixkm,jykm,cubic=-.5) & sob=sobs(inrge)
wmsda=sqrt(umsa^2+vmsa^2)

;...evaluate the average absolute errorsf
abseums=total(abs(umsa-ums))/n_elements(inrge)
absevms=total(abs(vmsa-vms))/n_elements(inrge)
absewms=total(abs(wmsa-wms))/n_elements(inrge)
absewmsd=total(abs(wmsda-wms))/n_elements(inrge)

;...write the file which gives the observed and analyzed winds at each observation site
openw,lu,'statfile.dat',/GET_LUN
fmt='(i4,i4,6f8.3)'
printf,lu,'   K  NTP   XKM     YKM     UMS     UMSA    VMS     VMSA'
;for k=1,n_elements(inrge)-1 do $ change done on 11/14 by Bachir Annane
for k=long(1),long(n_elements(inrge)-1) do $
printf,lu,format=fmt,k,ntp(k),xkm(k),ykm(k),ums(k),umsa(k),vms(k),vmsa(k)
close,lu & free_lun,lu
;END STATFILE.M

;....COMPUTE MEAN AND RMS
;... compute errors for different kind of storms
openw,lu,'all_stat_error.dat',/GET_LUN
;... this part is for speed
wspd=sqrt((ums)^2+(vms)^2)
wspda=sqrt((umsa)^2+(vmsa)^2)
diff=wspda-wspd
;... this part is for direction
azima=180*atan(umsa,vmsa)/!pi
nazima=azima+180
azim=180*atan(ums,vms)/!pi
nazim=azim+180
diff_dir=nazima-nazim
kk=size(nazim,/n_dim)
anom=where(diff_dir ge 290.0,ict) & if (ict gt 0) then diff_dir(anom)=360.0 - diff_dir(anom)
n_anom=where(diff_dir le -290.0,ict) & if (ict gt 0) then diff_dir(n_anom)=-1.0*(360.0 + diff_dir(n_anom))
;...case of a hurricane
hrcn=where(wspda ge 20.0,ict1) 
if (ict1 gt 0) then begin $
mn_wspd_hrcn=1.94*total(diff(hrcn))/n_elements(hrcn) &$
mn_dir_hrcn=total(diff_dir(hrcn))/n_elements(hrcn) &$
rms_wspd_hrcn=1.94*sqrt(total(diff(hrcn)^2)/n_elements(hrcn)) &$
rms_dir_hrcn=sqrt(total(diff_dir(hrcn)^2)/n_elements(hrcn)) &endif
;...case  of a tropical storm
trps=where(wspda ge 10.0,ict0) 
if (ict0 gt 0) then begin $
mn_wspd_trps=1.94*total(diff(trps))/n_elements(trps) &$
mn_dir_trps=total(diff_dir(trps))/n_elements(trps) &$
rms_wspd_trps=1.94*sqrt(total(diff(trps)^2)/n_elements(trps)) &$
rms_dir_trps=sqrt(total(diff_dir(trps)^2)/n_elements(trps)) &endif
;...case of a depression
dprs=where(wspda ge 5.0,ict2) 
if (ict2 gt 0) then begin $
mn_wspd_dprs=1.94*total(diff(dprs))/n_elements(dprs) &$
mn_dir_dprs=total(diff_dir(dprs))/n_elements(dprs) &$
rms_wspd_dprs=1.94*sqrt(total(diff(dprs)^2)/n_elements(dprs)) &$
rms_dir_dprs=sqrt(total(diff_dir(dprs)^2)/n_elements(dprs)) &endif
;...Here we consider different kind of scenarios;
if ( max(wspda) ge 33.4) then begin $
mn_err=mn_wspd_hrcn & mndr_err=mn_dir_hrcn & rms_wsp=rms_wspd_hrcn &$
rms_dir=rms_dir_hrcn &endif
if ( max(wspda) ge 18.0) then begin $
mn_err=mn_wspd_trps & mndr_err=mn_dir_trps & rms_wsp=rms_wspd_trps &$
rms_dir=rms_dir_trps &endif
if ( max(wspda) lt 18.0) then begin $
mn_err=mn_wspd_dprs & mndr_err=mn_dir_dprs & rms_wsp=rms_wspd_dprs &$
rms_dir=rms_dir_dprs &endif
;... write all the errors to a file 
if (ict1 gt 0) then printf,lu,mn_wspd_hrcn,mn_dir_hrcn else printf,lu,'      NA      NA'
if (ict0 gt 0) then printf,lu,mn_wspd_trps,mn_dir_trps else printf,lu,'      NA      NA'
if (ict2 gt 0) then printf,lu,mn_wspd_dprs,mn_dir_dprs else printf,lu,'      NA      NA'
if (ict1 gt 0) then printf,lu,rms_wspd_hrcn,rms_dir_hrcn else printf,lu,'      NA      NA'
if (ict0 gt 0) then printf,lu,rms_wspd_trps,rms_dir_trps else printf,lu,'      NA      NA'
if (ict2 gt 0) then printf,lu,rms_wspd_dprs,rms_dir_dprs else printf,lu,'      NA      NA'
close,lu & free_lun,lu
;
openw,lu1,'stat_error_knots.dat',/GET_LUN
printf,lu1,mn_err,mndr_err
printf,lu1,rms_wsp,rms_dir
close,lu1 & free_lun,lu1
;
openw,lu1,'stat_error_ms.dat',/GET_LUN
printf,lu1,mn_err/1.94,mndr_err
printf,lu1,rms_wsp/1.94,rms_dir
close,lu1 & free_lun,lu1
;
;... END OF COMPUTATION OF THE MEAN AND RMS.

;RCTOPLR.M
;...evaluate the analyzed wind components on a polar grid with a 10 degree azimuthal spacing
;   and a radial spacing of 1 RMW out to a distance of 20 RMW
restore,'xypl20.idl'
ixx=n_elements(xmkm) & jxx=n_elements(ynkm)
xgrid=float(zgrid) & ygrid=imaginary(zgrid)
xpl=rxobs(0)*xnpl & ypl=rxobs(0)*ynpl
ipl=xpl/max(xgrid) & jpl=ypl/max(ygrid)
ipl=80*(ipl+1) & jpl=80*(jpl+1)
zpl=interpolate(zmrg,ipl,jpl,cubic=-.5)
wpl=float(sqrt(zpl*conj(zpl)))
;END RCTOPLR.M

;...Hovmoeller part here
hov=fltarr(1,201) & for i=0,200 do hov(0,i)=total(wpl(i,*))/36.0
save,rxmeso,rxobs,zpl,hov,filename='hov.dl'
;

;DISPLAY.M
;...prepare high resolution display arrays
info=size(zuvbc) & imn=indgen(4*info(1)-3)/4.
zuvbcx=interpolate(zuvbc,imn,imn,/grid,cubic=-.5)
wind=float(sqrt(zuvbcx*conj(zuvbcx)))

;... high resolution coordinate arrays
xm=imn*dxyk & xm=xm-xm(n_elements(xm)-1)/2 
ijcn=where(abs(xm) eq min(abs(xm))) & ijcn=ijcn(0)
iwmin=where(wind eq min(wind(ijcn-3:ijcn+3,ijcn-3:ijcn+3)))
jcn=iwmin(0)/n_elements(xm) & icn=iwmin(0)-jcn*n_elements(xm)
xm=xm-xm(icn) & yn=xm-xm(jcn)
XYTOLL,1000*xm,1000*yn,elonx,nlatx,0.,0.
elonx=(elonx+nlatx)/2. & nlatx=elonx+gglo & elonx=elonx+elo

;... form high resolution landmask
LANDMASKI,elonx,nlatx,lndmskx ; current device is Z-buffer

;... open terrain conversion over land
ZUV2ZOT,zuvbcx,zotbcx

;... merge marine and land analyses
ZUVMRG,zuvbcx,zotbcx,lndmskx,zmergx
if levelzo ne 19 then zmergx=zuvbcx
wind=float(sqrt(zmergx*conj(zmergx)))
dxy=250*dxyk
;END DISPLAY.M

;LLSTAR.M
;...form latitude and longitude of polar grid sites.
xytoll,1000*xpl,1000*ypl,elpl,nlpl,elo,gglo
elpl=reform(elpl,201,37) & nlpl=reform(nlpl,201,37)
;LLSTAR.M

;COLORPS.M
;PLOTSETC.M
;...acquire the color table and initialize postscript rendering
set_plot,'Z'
restore,'colors1.idl' & tvlct,red,green,blue
set_plot,'ps',/copy
device,/close
device,xoffset=1,yoffset=3,xsize=20*cos(!pi*gglo/180.),ysize=20,/portrait,/color
!x.range=0 &!y.range=0 & !x.style=1 &!y.style=1
;END PLOTSETC.M

;...render the windfield in three different resolutions
;...FOUR DEGREE PLOT
device,filename='col04deg.ps'
ndg=2 & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close
;...FOUR DEGREE PLOT
device,filename='col04deg.ps'
ndg=2 & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close
;...TWO DEGREE PLOT
device,filename='col02deg.ps'
ndg=1 & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close
wind=wind > 5.
;...EIGHT DEGREE PLOT
device,filename='col08deg.ps'
ndg=n_big & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close

;...TEN DEGREE PLOT
device,filename='col10deg.ps'
ndg=5 & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close

;...14 DEGREES PLOT
device,filename='col14deg.ps'
ndg=7 & CLINPS,zmergx,wind,elo,gglo,elonx,nlatx,ndg,qrad,colors,levels,zpl,wpl,elpl,nlpl
device,/close

;...plot vorticity and divergence
;@vort_plot.m
;@div_plot.m

;...create google products
@google_product.m

;END COLORPS.M

END ;HWIND.PRO
